//! [PUT /_matrix/federation/v1/invite/{roomId}/{eventId}](https://matrix.org/docs/spec/server_server/r0.1.4#put-matrix-federation-v1-invite-roomid-eventid)

use js_int::UInt;
use ruma_api::ruma_api;
use ruma_events::{room::member::MemberEventContent, EventType};
use ruma_identifiers::{EventId, RoomId, ServerName, UserId};
use serde::{Deserialize, Serialize};

use super::{InviteEvent, StrippedState};

ruma_api! {
    metadata: {
        description: "Invites a remote user to a room.",
        method: PUT,
        name: "create_invite",
        path: "/_matrix/federation/v1/invite/:room_id/:event_id",
        rate_limited: false,
        requires_authentication: true,
    }

    #[non_exhaustive]
    request: {
        /// The room ID that the user is being invited to.
        #[ruma_api(path)]
        pub room_id: RoomId,

        /// The event ID for the invite event, generated by the inviting server.
        #[ruma_api(path)]
        pub event_id: EventId,

        /// The matrix ID of the user who sent the original `m.room.third_party_invite`.
        pub sender: UserId,

        /// The name of the inviting homeserver.
        pub origin: Box<ServerName>,

        /// A timestamp added by the inviting homeserver.
        pub origin_server_ts: UInt,

        /// The value `m.room.member`.
        #[serde(rename = "type")]
        pub kind: EventType,

        /// The user ID of the invited member.
        pub state_key: UserId,

        /// The content of the event.
        pub content: MemberEventContent,

        /// Information included alongside the event that is not signed.
        pub unsigned: UnsignedEventContent,
    }

    #[non_exhaustive]
    response: {
        /// The response invite event
        #[ruma_api(body)]
        #[serde(with = "crate::serde::invite_response")]
        event: InviteEvent,
    }
}

/// Information included alongside an event that is not signed.
#[derive(Clone, Debug, Serialize, Deserialize)]
pub struct UnsignedEventContent {
    /// An optional list of simplified events to help the receiver of the invite identify the room.
    /// The recommended events to include are the join rules, canonical alias, avatar, and name of
    /// the room.
    pub invite_room_state: Vec<StrippedState>,
}

/// Initial set of fields of `Request`.
pub struct RequestInit {
    /// The room ID that the user is being invited to.
    pub room_id: RoomId,

    /// The event ID for the invite event, generated by the inviting server.
    pub event_id: EventId,

    /// The matrix ID of the user who sent the original `m.room.third_party_invite`.
    pub sender: UserId,

    /// The name of the inviting homeserver.
    pub origin: Box<ServerName>,

    /// A timestamp added by the inviting homeserver.
    pub origin_server_ts: UInt,

    /// The user ID of the invited member.
    pub state_key: UserId,

    /// The content of the event.
    pub content: MemberEventContent,

    /// Information included alongside the event that is not signed.
    pub unsigned: UnsignedEventContent,
}

impl From<RequestInit> for Request {
    /// Creates a new `Request` with the given parameters.
    fn from(init: RequestInit) -> Self {
        Self {
            room_id: init.room_id,
            event_id: init.event_id,
            sender: init.sender,
            origin: init.origin,
            origin_server_ts: init.origin_server_ts,
            kind: EventType::RoomMember,
            state_key: init.state_key,
            content: init.content,
            unsigned: init.unsigned,
        }
    }
}

impl Response {
    /// Creates a new `Response` with the given invite event.
    pub fn new(event: InviteEvent) -> Self {
        Self { event }
    }
}
